;+
; Procedure: plotxy
;
; Purpose: Takes an array of 3-d(Nx3) vectors or tplot variable
;          and plots them using 2-d plots to help visualize them. 
;          It can also take an MxNx3 array tplot variable storing 
;          an MxNx3 array which represents a set of M lines with N
;          points 
;
;         If a tplot variable is used, this routine will check the
;         current timespan and only display values within the
;         specified range(this entails clipping on the N dimension)
;
;         To avoid distortion you should try to make the window as
;         square as possible. If this program is making the window it 
;         will calculate a square plot window based upon the available
;         data
;
;         Can also accept an Nx2 or an MxNx2 element array...if this
;         is done the versus argument should not use a custom
;         designation or the z-axis, as it assumes 2-d vectors are in
;         the x-y plane, and thus will distort the vectors upon projection.
;
;          
;         plotxy/tplotxy plots can be interleaved on the same window
;         with plotxyz plots
;
;         Calling tplotxy with no arguments to redraw the entire
;         window(including plotxyz plots)
;
;         ***************************************************
;
;         Using custom axes:  If you use two vectors to define custom
;         axes, the procedure will generate a plot of the data vectors
;         projected into a plane defined by the span of the two custom
;         vectors.  The x-axis will be the first vector, the y axis
;         will be the second vector.  This means that if the custom
;         vectors are not orthogonal the plot will show a distortion.
;         You can think of this as plotting along a plane that slices
;         though the 3-d space.
; 
;         ********************************
;         Plot windows and panels:
;         using, /overplot,/addpanel,/noisotropic and multi=
;
;         To put multiple panels in a window first call
;         plotxy with the multi keyword.  It will either
;         plot in which ever window is your current one, or
;         create a new one if no window exists or if you 
;         request the use of a nonexistent window.
;
;         During this first call you may want to specify things
;         like wtitle,xsize,ysize,window...in addition to your normal
;         plotting options. However,Calling window options will interfere with
;         the creation of postscripts. 
;
;         multi specifies the plot window panel layout.
;         So if you set multi='3,2' you will get 6 plots
;         in your window with a layout like:
;           -------
;           |x x x|
;           |x x x|      
;           -------
;
;         Each panel will have dimensions x number of pixels = 1/3 *
;         xsize of window and y number of pixels = 1/2 * ysize of window.
;
;         Your first call should also specify the layout of your 
;         first panel. To add to that panel use the /overplot keyword.
;         
;         When you use the /add keyword the program will move on to
;         the next panel within the plot window and you should add
;         options to specify the layout of that panel. 
; 
;         If you set the xmargin or ymargin keyword the margin will be
;         relative to the overall size of that panel. When using the
;         not using the noisotropic keyword the procedure will make
;         each axis vary over the same range AND make the
;         largest possible square window given the size of the panel
;         and the sizes of the margins you have provided, if possible.
;         In some cases when ranges are set explictly the plot must
;         be rectangular.
;
;         An entire plot window must be filled in sequence, if you move
;         on to a new window you will not be able to go back to the
;         previous window without restarting.
;
;         If you call plotxy with no arguments it will redraw the
;         entire window including all panels and overplots.  If you
;         resize the window before calling with new arguments it
;         will redraw the isotropic panels as the largest possible
;         squares.  This comes at a cost of storing copies of the 
;         commands and data you made in memory.  If you need to save
;         memory you can call the function with the /memsave argument,
;         but then redraws will be done using hardware and window resizes
;         can distort isotropic plots.
;
;         NOTE TO PROGRAMMERS:
;           Information about plotting for plotxy is stored in
;           the global variable !TPLOTXY, this includes
;           information about the layout of the plot window
;           which panel it is currently working on, and the 
;           sequence of commands used to generate current plot window
;           so that it can regenerate the plotwindow when called
;           with no arguments. This variable also stores information
;           used by the plotxyz function so spectrographic xyz plots 
;           can be interleaved with xy line plots.
;  
;
;Example: a = [[dindgen(10)],[dindgen(10)],[dindgen(10)]]
;         get_data,'thb_state_pos',data=d
;         dat=d.y
;         plotxy
;         plotxy,a
;         plotxy,a,versus='yzr'
;         plotxy,dat,versus='cc',custom=transpose([[1,1,0],[0,0,1]])
;         plotxy,dat,versus='xryz',xrange=[0,10],yrange=[0,10]
;
; Inputs: vectors(optional): an Nx3,MxNx3,Nx2, or MxNx2 list
;
; Keywords:
;
;         versus(optional): specify the projection to be used, can be
;         'xx','xy','xz','yx','yy','yz','zx','zy','zz','cc' you can also
;         follow a letter with an 'r' to reverse the axis(goes from
;         positive to negative instead of from negative to positive) 
;         if you specify 'cc','crc'...that indicate you want to use a
;         custom projection
;         example: 'xry' will be an xy plot with the maximum x value
;         listed on the left and the minimum on the right 
;         (default:'xy')
;
;         custom(optional): set this variable to a
;         2x3 matrix whose columns define a plane in 3-d space, to define a
;         custom projection. In other words the 2-d plot will be a
;         plot of the vectors passed into plotxy when they are
;         projected into a plane defined by
;         span(custom[0,*],custom[1,*]). (span is defined as the set
;         of all the linear combinations of two vectors, or 
;         span(x,y) = {mx+ny:m = element of the reals, n = element of
;         the reals} The vectors used to define this plane will be
;         relative to whatever 3-d coordinate system the input vector
;         data is in.
;         So if the call:
;         tplotxy,'somedata',versus='cc',custom=transpose([[1,1,0],[0,0,1])
;         is made, the plot generated will be of the vectors closest
;         to the data vectors that are inside a vertical plane whose
;         intersection with the x-y plane forms a line y=x.
;         tplotxy,'somedata',versus='cc',custom=transpose([[1,0,0],[0,1,0])
;         is effectively the same as:
;         tplotxy,'somedata',versus='xy'
;  
;         overplot(optional): set this keyword if you want to plot
;         on the last plot and panel that you plotted on
;
;         addpanel(optional): set this keyword if you want to plot on a new
;         panel within the same plot window as where you last
;         plotted. This will go to the next column first and if it is
;         at the end of a row, to the next row.
;
;         multi(optional): set this keyword to a string that 
;         specifies the layout of panels within a plotwindow.  Set
;         this keyword only the first time you call tplotxy for a 
;         given plotwindow.  Each time you set it, the previous
;         contents of the window will be erased. You can separate
;         the two elements with a variety of different delimiters
;         The first element is columns left to right, the second rows
;         top to bottom. Append an 'r' to the elements to have it
;         reverse the direction of panel application
;         Examples:  multi= '2 3' 
;                    multi= '5r,7'
;                    multi=' 1:6r'
;                     .....
;
;         noisotropic(optional): set this keyword if you don't want the
;         scaling of both axes to be the same and the space to
;         be perspective corrected so that a cm of y unit takes
;         up the same space on the screen as a cm of x unit
;
;         memsave(optional): set this keyword to request command
;         copies not be saved and redraws be done without maintaining
;         square isotropic plots.  Setting this option can potentially
;         save quite a lot of memory.
;
;         linestyle(optional):set this to change the linestyle used
;         0 = default,1=dotted=,2=dashed,3=dash dot,4=dash dot
;         dot,5=long dashes
;
;         xrange(optional): set this to a 2 element array to specify
;         the min and max for the first axis(x) of the 2-d plot
;
;         yrange(optional): set this to a 2 element array to specify
;         the min and max for the second axis(y) of the 2-d plot
;
;         pstart(optional): set this keyword to a number representing
;         the symbol you would like to start lines with. (This works
;         like the idl psym keyword, but only for the first symbol
;         in a line being plotted)
;
;         pstop(optional): set this keyword to a number representing
;         the symbol you would like to end lines with. (This works
;         like the idl psym keyword, but only for the last symbol
;         in a line being plotted)
;
;         psym(optional): use this to plot the line using a symbol
;         rather than a line.
;
;         symsize(optional): specify the size of the start and end
;         symbol or psym. (default:1.0)
;
;         WARNING: setting any of the 4 windowing options below
;         will interfere with postscripts
;
;         window(optional):specify the window for
;         output, if overplot is not specified, it will
;         always recreate the window so it can attempt to make
;         the window square (default:current)
;
;         xsize(optional):specify the xsize of the window in
;         pixels(default: current)
;
;         ysize(optional):specify the ysize of the window in pixels
;         (default: current)
;
;         wtitle(optional):the title you would like the window to have
;
;         colors(optional): if vectors in Nx3 colors should contain a
;         single element(name like 'r' or index like 2), if vectors is
;         an MxNx3 then it can contain a single element or M elements
;
;         xmargin(optional): set this option to a two element array
;         specifing the size of the margin of the current panel
;         relative to the size of overall panel on the x dimension.
;         Values range from 0.0(no margin) to 1.0(all margin)
;         The first element of the array is the left margin
;         The second element is the right margin
;
;         ymargin(optional): set this option to a two element array
;         specifing the size of the margin of the current panel
;         relative to the size of overall panel on the y dimension.
;         Values range from 0.0(no margin) to 1.0(all margin)
;         The first element of the array is the bottom margin and
;         The second element of the array is the top margin
;
;         xlog(optional): set the x scale to be logarithmic
;
;         ylog(optional): set the y scale to be logarithmic
;
;         xtitle(optional): set the xtitle for the plot
;
;         ytitle(optional): set the ytitle for the plot
;
;         grid(optional): set this to 1 to have the procedure
;         generate a grid rather than normal tickmarks
;
;         units(optional): set this if you want this unit label
;         appended to both axis titles.(This will be ignored if 
;         you set the xtitle or ytitle explictly) 
;
;         labels(optional): set this if you want to use the axis
;         labels array from limits/dlimits of a tvar.(This will be 
;         ignored if you set the xtitle or ytitle explictly) 
;
;         markends(optional): set this if you want to mark the very
;         edges of your plot axis with data labels. This means the
;         numerical values of the maximum x tick,minimum x tick,
;         maximum y tick, and minimum y tick will be marked. Note that
;         an extra blank page may be created in any postscripts you
;         generate when using this option. 
; 
;             
;         replot(internal): this option is used in recursive calls
;         by the routine to itself and should never be set by the
;         user  
;       
;         This function also takes normal idl keywords that effect
;         plotting style(things like xtitle,ytitle....etc..)
;
;  SEE ALSO:
;    plotxyz,tplotxy,thm_crib_tplotxy,thm_crib_plotxy,thm_crib_plotxyz
;
; $LastChangedBy: pcruce $
; $LastChangedDate: 2008-01-16 16:54:40 -0800 (Wed, 16 Jan 2008) $
; $LastChangedRevision: 2283 $
; $URL: svn+ssh://thmsvn@ambrosia.ssl.berkeley.edu/repos/ssl_general/trunk/tplot/tplotxy.pro $
;-

;HELPER FUNCTION
;takes a matrix whose columns define a plane
;and an Nx3 series of points
;returns an Nx2 series of points generated by projecting x into a
function p3p_project, a, x

compile_opt hidden, idl2

dims = size(x, /dimensions)

out = make_array(dims[0], 2, /double, value = !values.d_nan)

idx1 = where(finite(x[*, 0]))
idx2 = where(finite(x[*, 1]))
idx3 = where(finite(x[*, 2]))

;if NaN's are not distributed symmetrically among x,y,z
;intersection is calculated, to identify vectors that contain no NaNs
idxt = ssl_set_intersection(idx1, idx2)

idxt = ssl_set_intersection(idx3, idxt)

;if there are no vectors that contain no NaNs return a vector of all NaNs
if idxt[0] eq -1 then return, out

;calculate the matrix that will project into specified plane
p = invert(transpose(a) ## a) ## transpose(a)

;perform projection
vals = p ## x[idxt, *]

out[idxt, *] = vals

return, out

end

;HELPER FUNCTION
;takes a plot axis string and gets the elements corresponding to the
;letter in the string,passes back the requested axis in ele, the
;default title, whether there was an error, and whether axis reversal
;was requested
pro p3p_parse_elements,string, element, vectors, custom, ele = ele, title = title, error = error,$
                       reverse = reverse, unit=unit, label=label,dimtitle = dimtitle

  compile_opt hidden,idl2

  error = 1

  reverse = 0

  dimtitle = ''

  if(element eq 1) then begin

    char = strmid(string, 0, 1)
    if strmid(string, 1, 1) eq 'r' then reverse = 1

  endif else if element eq 2 then begin

    len = strlen(string)
    char = strmid(string, len-1, 1)

    if(char eq 'r') then begin
      reverse = 1
      char = strmid(string, len-2, 1)
    endif

  endif

  title = string

  if(char eq 'x') then begin 
     ele = vectors[*, *, 0] 

     if keyword_set(label) && n_elements(label) eq 3 then begin
        dimtitle = label[0]
     endif else begin
        dimtitle = 'X'
     endelse

  endif else if(char eq 'y') then begin 
    ele = vectors[*, *, 1] 
  
    if keyword_set(label) && n_elements(label) eq 3 then begin
       dimtitle = label[1]
    endif else begin
       dimtitle = 'Y'
    endelse


  endif else if(char eq 'z') then begin 
    ele = vectors[*, *, 2] 
  
    if keyword_set(label)  && n_elements(label) eq 3 then begin
       dimtitle = label[2]
    endif else begin
       dimtitle = 'Z'
    endelse


  endif else if(char eq 'c') then begin
    
    if not keyword_set(custom) then begin
      message, /continue, 'custom axes not set where custom axes specified'
      return
    endif

    dims =  size(vectors, /dimensions)

    ele = dblarr(dims[0], dims[1])
    
    for i = 0, dims[0]-1 do begin

      vs = p3p_project(custom, reform(vectors[i, *, *]))

      ele[i, *] = reform(vs[*, element-1])

    endfor

    title = strcompress('[' + strjoin(reform(string(custom[0, *])), ',') + $
            '] vs!C[' +  strjoin(reform(string(custom[1, *])), ',') + ']')

    dimtitle = strcompress('[' +  strjoin(reform(string(custom[element-1, *])), ',') + ']')

  endif else begin 
    message, /continue, 'Illegal plot axis string passed to plot3project'
    return
  endelse

  if keyword_set(unit) then begin
    dimtitle += ' ' + unit
  endif   


  error = 0

  return

 end

;HELPER FUNCTION
;set window up
pro p3p_set_window,window,xsize,ysize,wtitle

compile_opt hidden,idl2

;if the window has not yet been generated or no more
;windows are available


device,window_state=wlist

if keyword_set(window) then begin

  ;if the window doesn't really exist create it
  if window ge n_elements(wlist) || window lt 0 then $
    message,'You passed an out of range window value' $
  else if wlist[window] eq 0 then $
    window,window,xsize=640,ysize=512 $
  else $
    wset,window 
    
endif else if !D.window eq -1 then begin
  window = 0
  xsize = 640
  ysize = 512
endif else $
  window = !D.window

if not keyword_set(xsize) then begin
  xs = !D.X_SIZE
endif else begin
  xs = xsize
endelse

if not keyword_set(ysize) then begin
  ys = !D.Y_SIZE
endif else begin
  ys = ysize
endelse

if not keyword_set(wtitle) then begin 
  wt = strcompress('IDL ' + string(window))
endif else begin
  wt = wtitle
endelse

window,window,xsize=xs,ysize=ys,title=wt,retain=2

end

;helper function, intializes the tplotxy system variable
pro p3p_set_state,multi

  compile_opt idl2,hidden

  plotlist = 0

  if keyword_set(multi) then begin

     args = strsplit(multi,' *[ ,/:;\.\\] *',/extract,/fold_case,/regex,count=c)

     if c ne 2 then begin
        message,'illegal multi string "' + multi + '"'
     endif

     if stregex(args[0],'r',/boolean) then begin
        revcols = 1
     endif else begin
        revcols = 0
     endelse

     if stregex(args[1],'r',/boolean) then begin
        revrows = 1
     endif else begin
        revrows = 0
     endelse

     cols = long(stregex(args[0],'[0-9]*',/extract))

     if cols eq 0 then begin
        message,'error parsing multi cols: "' + args[0] + '"'
     endif 

     rows = long(stregex(args[1],'[0-9]*',/extract))
     
     if cols eq 0 then begin
        message,'error parsing multi rows: "' + args[1] + '"'
     endif 

  endif else begin
    
     revcols = 0
     revrows = 0
     cols = 1
     rows = 1
 
  endelse

  DEFSYSV,'!tplotxy',exists=bool

  ;free any old memory
  if bool && is_struct(*(!tplotxy.plotvec)) then begin
     
     plotvec = !tplotxy.plotvec

     t=csvector(*plotvec,/free)
     ptr_free,plotvec

  endif

  tplotxyval = { rows:rows,$
                 revrows:revrows,$
                 cols:cols,$
                 revcols:revcols,$
                 current:0,$
                 plotvec:ptr_new(csvector('start')) }


  DEFSYSV,'!tplotxy',tplotxyval

end

;determines the position of the current panel
;using the margin information and the tplotxy information
function p3p_get_pos,xmargin,ymargin,square

  compile_opt idl2,hidden

  ;validate inputs
  if keyword_set(xmargin) then begin

     if n_elements(xmargin) ne 2 then begin
        message,'malformed x margin'
     endif

     if ~is_num(xmargin,/real) then begin
        message,'x margin must contain real number'
     endif

     id = where(xmargin gt 1 or xmargin lt 0) 

     if id[0] ne -1 then begin
        message,'x margin must be in the range [0,1]'
     endif

  endif else begin

     xmargin = [.15,.05]

  endelse
              
  if keyword_set(ymargin) then begin

     if n_elements(ymargin) ne 2 then begin
        message,'malformed y margin'
     endif
     
     if ~is_num(ymargin,/real) then begin
        message,'y margin must contain real number'
     endif

     id = where(ymargin gt 1 or ymargin lt 0) 

     if id[0] ne -1 then begin
        message,'y margin must be in the range [0,1]'
     endif

  endif else begin

     ymargin = [.1,.075]

  endelse

  ;verify that the current panel is not too large
  
  if !tplotxy.current ge (!tplotxy.rows * !tplotxy.cols) then begin
     message,'no more panels available in current layout'
  endif

  xpanel_size = 1.0/!tplotxy.cols
  ypanel_size = 1.0/!tplotxy.rows

  if !tplotxy.revcols eq 0 then begin
     xpanel_cur = !tplotxy.current mod !tplotxy.cols
  endif else begin
     xpanel_cur = !tplotxy.cols - 1 - (!tplotxy.current mod !tplotxy.cols)
  endelse 

  if !tplotxy.revrows ne 0 then begin
     ypanel_cur = !tplotxy.current / !tplotxy.cols
  endif else begin
     ypanel_cur = !tplotxy.rows - 1 - (!tplotxy.current / !tplotxy.cols)
  endelse

  ;coordinates
  x1 = xpanel_cur * xpanel_size + xpanel_size*xmargin[0]
  x2 = (xpanel_cur+1) * xpanel_size - xpanel_size*xmargin[1]

  y1 = ypanel_cur * ypanel_size + ypanel_size*ymargin[0]
  y2 = (ypanel_cur+1) * ypanel_size - ypanel_size*ymargin[1]

  ;make plots square
  if keyword_set(square) then begin
     

     ;pixel ratio...for screens that have pixels that are not square

     ;plot size
     ;multiply by pix number of a side
     ;and divide by pix # / cm
     ;to normalize side comparisons
     xplotsize = (!D.x_size*(x2-x1))/!D.x_px_cm
  
     yplotsize = (!D.y_size*(y2-y1))/!D.y_px_cm


     if xplotsize gt yplotsize then begin

        x2 += (yplotsize - xplotsize)*!D.x_px_cm/!D.x_size

     endif else begin 

        y2 += (xplotsize - yplotsize)*!D.y_px_cm/!D.y_size

     endelse
  endif

  return, [x1,y1,x2,y2]

end

function p3p_fix_ticks,tget,mint,maxt,log

  compile_opt idl2,hidden

   nt = n_elements(tget)

   tget[0] = mint
   tget[nt-1]=maxt

   ;check to make sure that the added ticks don't collide
   return,p3p_tick_int(p3p_tick_int(tget,log,0),log,1)

end

function p3p_tick_int,ticks,log,first

  tolerance=.4
  
  n = n_elements(ticks)

  if n lt 3 then return, ticks

  if keyword_set(log) then begin
     ticksv = alog(ticks) 
  endif else begin
     ticksv = ticks
  endelse

  if first then begin
     dif = ticksv[1]-ticksv[0]
     rest = ticksv[1:n-1]
  endif else begin
     dif = ticksv[n-1]-ticksv[n-2]
     rest = ticksv[0:n-2]
  endelse

  avg_dif = total(rest[1:n-2]-rest[0:n-3])/(n-2)

  if dif/avg_dif lt tolerance then begin

     if first then begin
        return,[ticks[0],ticks[2:n-1]]
     endif else begin
        return,[ticks[0:n-3],ticks[n-1]]
     endelse

  endif else begin

     return,ticks

  endelse

end

;adds the information to repeat the previous tplotxy call to the 
;tplotxy global variable
pro p3p_push_state,vectors,_extra=ex

  compile_opt idl2,hidden

  plotvec = *(!tplotxy.plotvec)

  ptr_free,!tplotxy.plotvec

  state = {vectors:vectors,ex:ex}

  func = {func:'plotxy',state:state}

  !tplotxy.plotvec = ptr_new(csvector(func,plotvec))

end

pro p3p_replot

  compile_opt idl2,hidden

  !tplotxy.current = 0

  plotvec = *(!tplotxy.plotvec)

  len = csvector(plotvec,/length)

  for i = 1,len-1 do begin

     c = csvector(i,plotvec,/read)

     state=c.state

     if c.func eq 'plotxy' then begin

        plotxy, state.vectors,replot=1,_extra=state.ex

     endif else if c.func eq 'plotxyz' then begin

        plotxyz,state.x,state.y,state.z,replot=1,_extra=state.ex

     endif else begin
        
        message,'unrecognized replot function'

     endelse

  endfor

end

;main function
pro plotxy, vectors, versus=versus, symsize=symsize, custom = custom,title=title,overplot=overplot,$
 addpanel=addpanel,multi=multi,memsave=memsave,noisotropic=noisotropic,linestyle=linestyle, xrange = xrange,$
 yrange = yrange, pstart=pstart,pstop=pstop, window = window, xsize = xsize, ysize = ysize, xmargin = xmargin, ymargin = ymargin,$
 wtitle=wtitle,xtitle=xtitle,ytitle=ytitle,colors=colors,replot=replot,xlog=xlog,ylog=ylog,units=units,labels=labels, $
 grid=grid,markends=markends,marks=marks,_extra = _extra

compile_opt idl2

;adds a margin
;plotsize = 1.0D/8.0D

;set defaults and check some input invariants
if ~keyword_set(symsize) then symsize=1.0D

if ~keyword_set(versus) then versus = 'xy'

if keyword_set(xrange) && n_elements(xrange) ne 2 then message, 'xrange must have two elements'

if keyword_set(yrange) && n_elements(yrange) ne 2 then message, 'yrange must have two elements'

if keyword_set(overplot) && keyword_set(addpanel) then begin

   message,'cannot set addpanel and overplot at the same time'

endif

if keyword_set(marks) then begin
  message,/continue,'marks keyword has been replaced. please use pstart/pstop keywords instead'
endif  

if ~keyword_set(noisotropic) then begin
   isotropic = 1
endif else begin
   isotropic = 0
endelse

;replot call
if ~keyword_set(vectors) then begin

   p3p_replot

   return

endif

;first call, do general setup stuff
if ~keyword_set(overplot) && ~keyword_set(addpanel) then begin
   
   if keyword_set(window) or keyword_set(xsize) or keyword_set(ysize) or keyword_set(wtitle) then begin
   
     p3p_set_window,window,xsize,ysize,wtitle
     
   endif 

   ;if this isn't a recursive call making use of the state information
   ;then we can safely reset the state information
   if ~keyword_set(replot) then begin

      p3p_set_state,multi

   endif

endif

if(size(vectors, /type) eq 7) then $
   message,'cannot take a string argument, use tplotxy instead' $
else $
   vecs=vectors

dims = size(vecs, /dimensions)

if(n_elements(dims) ne 2 && n_elements(dims) ne 3) then message, 'vector argument must be a 2 or 3 dimensional array'

;if a list of 2-d points is passed in turn it into
;a list of 3-d points so all cases can be handled using the same code
if(dims[n_elements(dims)-1]) eq 2 then begin
   dims[n_elements(dims)-1] = 3
   temp = dblarr(dims)

   if(n_elements(dims) eq 2) then begin
      temp[*,0] = vecs[*,0]
      temp[*,1] = vecs[*,1]
   endif else begin
      temp[*,*,0] = vecs[*,*,0]
      temp[*,*,1] = vecs[*,*,1]
   endelse

   vecs = temp
endif

if(dims[n_elements(dims)-1] ne 3) then message, 'last dimension of vector argument must be size 3'

;if a 2-d argument is passed, make it 3-d so all cases can be handled
;using the same code
if(n_elements(dims) eq 2) then vecs = reform(vecs, [1, dims])

;get the axes as requested using the versus and custom arguments
if not keyword_set(title) then $
  p3p_parse_elements, versus, 1, vecs, custom, ele = ele1, title = title, error = error, reverse = rev1, unit=units, label=labels,dimtitle = dt $
 else $
  p3p_parse_elements, versus, 1, vecs, custom, ele = ele1, error = error, reverse = rev1, unit=units, label=labels,dimtitle = dt

if not keyword_set(xtitle) then begin
   
      xtitle = dt

endif


dt = 0

if error then return 
  
p3p_parse_elements, versus, 2, vecs, custom, ele = ele2, error = error, reverse = rev2, unit=units, label=labels,dimtitle = dt

if not keyword_set(ytitle) then begin
   
      ytitle = dt

endif

if error then return 

;either set range automatically or manually
if keyword_set(xrange) then begin
  
   min_x = xrange[0]
   max_x = xrange[1]
  
   if(min_x gt max_x) then begin
      message,'Illegal x range, min gt max'
   endif

endif else begin
  
   max_x = max(ele1)
   min_x = min(ele1)
   
   x_int = max_x - min_x

endelse

if keyword_set(yrange) then begin
  
   min_y = yrange[0]
   max_y = yrange[1]
  
endif else begin

   max_y = max(ele2)
   min_y = min(ele2)

   y_int = max_y - min_y

   if(min_y gt max_y) then begin
      message,'Illegal y range, min gt max'
   endif
   
endelse

if keyword_set(isotropic) && ~keyword_set(xrange) && ~keyword_set(yrange) then begin
   square = 1

endif else begin
   square = 0
endelse

;isotropic behavior must also be done manually
;this code ensures side length is equal, if ranges are
;not set manually
if keyword_set(square) then begin

   ;
   xs = abs(max_x - min_x)
   ys = abs(max_y - min_y)

   if xs gt ys then begin

     diff = xs - ys

      min_y -= diff/2.
      max_y += diff/2.

   endif else begin

     diff = ys - xs

      min_x -= diff/2.
      max_x += diff/2.

   endelse

endif

;switch min and max if reversal of axis was requested
if rev1 then begin
   t = min_x
   min_x = max_x
   max_x = t
endif

if rev2 then begin
   t = min_y
   min_y = max_y
   max_y = t
endif

dims = size(ele1, /dimensions)

if keyword_set(colors) then $
   if n_elements(colors) ne dims[0] then begin
   if n_elements(colors) ne 1 then $
      message,'number of colors does not match number of dimensions' $
   else $
      cols = replicate(get_colors(colors),dims[0]) 
endif else $
   cols = get_colors(colors)
  
if keyword_set(addpanel) then begin

   !tplotxy.current++
   noerase=1

endif

pos = p3p_get_pos(xmargin,ymargin,square)

;create blank plot
if ~keyword_set(overplot) then begin

   if keyword_set(grid) then begin
      ticklen = 1.0
   endif

   ;invisible plot used to get tick information so that end points
   ;can be manually changed.  This is a little bit of a hack, but
   ;it is way easier than trying to write an automatic tick creation
   ;function by hand, which I imagine could get quite messy
  
   if (keyword_set(xrange) || keyword_set(yrange)) && keyword_set(isotropic) then begin 
      iso = 1
   endif else begin
      iso = 0
   endelse
    
   if keyword_set(markends) then begin

      plot,[min_x,max_x],[min_y,max_y],xrange = [min_x, max_x], yrange = [min_y, max_y],color=!P.background,$
           xtick_get=xtget,ytick_get=ytget, pos=pos,_extra=_extra,/nodata,/noerase,xlog=xlog,ylog=ylog,isotropic=iso

      xticks= p3p_fix_ticks(xtget,min_x,max_x,xlog)

      yticks= p3p_fix_ticks(ytget,min_y,max_y,ylog)

      plot,[min_x,max_x],[min_y,max_y],xtickv=xticks,xticks=n_elements(xticks)-1,ytickv=yticks,yticks=n_elements(yticks)-1,$
           xrange=[min_x,max_x],yrange=[min_y,max_y],title=title,xtitle=xtitle,ytitle=ytitle, pos=pos, _extra = _extra,/nodata,$
           noerase=noerase,xlog=xlog,ylog=ylog,isotropic=iso,ticklen=ticklen

   endif else begin
      
      plot,[min_x,max_x],[min_y,max_y],xrange=[min_x,max_x],yrange=[min_y,max_y],title=title,xtitle=xtitle,ytitle=ytitle, pos=pos,$
           _extra = _extra,/nodata,noerase=noerase,xlog=xlog,ylog=ylog,isotropic=iso,ticklen=ticklen,xstyle=1,ystyle=1

   endelse


endif

for i = 0, dims[0]-1 do begin

;identify NaNs
   plot1 = reform(ele1[i, *])

   idx1 = where(finite(plot1))
    
   plot2 = reform(ele2[i, *])

   idx2 = where(finite(plot2))
    
   idxt = ssl_set_intersection(idx1, idx2)

   if(idxt[0] eq -1) then begin 
      message, /continue, 'cannot plot an line composed entirely of NaNs, skipping line'
    
      continue
   endif

;filter NaNs
   plot1 = plot1[idxt]
    
   plot2 = plot2[idxt]

   if keyword_set(cols) then co = cols[i]

   oplot, plot1, plot2, linestyle = linestyle,color=co,symsize=symsize, _extra = _extra

   if keyword_set(pstart) then begin

      ;mark start
      oplot, make_array(1, value = plot1[0]), make_array(1, value = plot2[0]), psym = pstart, symsize = symsize

   endif
   
   if keyword_set(pstop) then begin

      ;mark stop
      oplot, make_array(1, value = plot1[n_elements(plot1)-1]), make_array(1, value = plot2[n_elements(plot2)-1]),$
             psym = pstop, symsize = symsize
  
   endif

endfor

;push the state
;push only at the end so we can be sure
;command succeeded
if ~keyword_set(replot) and ~keyword_set(memsave) then begin

   p3p_push_state, vectors, versus=versus, symsize=symsize, custom = custom,title=title,overplot=overplot,$
                   addpanel=addpanel,multi=multi,memsave=memsave,noisotropic=noisotropic,linestyle=linestyle,$
                   xrange = xrange, yrange = yrange, pstart=pstart,pstop=pstop,  xmargin = xmargin, ymargin = ymargin, $
                   xtitle=xtitle,ytitle=ytitle,colors=colors,replot=replot,xlog=xlog,ylog=ylog,units=units,$
                   labels=labels,grid=grid,markends=markends,marks=marks,_extra=_extra

endif


end
