;---------------------------------------------------------------
; Despin routine for EFI high-res (eff/efp/efw) data.
;
; Inputs:
;         probe: 1-character string identifying which spacecraft
;      datatype: 3-character string identifying data type ('eff','efp' or 'efw')
;        offset: 3-element array: Ex, Ey, Ez offset
;          gain: 3-element array: E12, E34, E56 gain
;   boom_offset: 2-element array: E12 offset, E34 offset
;                  (leave undefined to use auto-offset)
;    tplot_name: String scalar containing the name of the TPLOT variable to
;                  operate on (regardless of PROBE and DATATYPE kwd's).
;
; Options:
;       notch: set equal to the # of harmonics to remove (including fundamental)
;               (i.e. notch=1 removes fundamental only, notch=2 also removes 1st harmonic)
;               (default=0 does not notch filter anything)
; tweak_gains: set flag to tweak the E12/E34 gains to minimize the spin tone
;     newname: If specified, the cleaned data is stored as a tplot
;                variable with name=newname instead of the default
;                'th'+sc+'_'+datatype+'_dsl'
;
; Outputs:
;    Creates a new tplot variable with the name 'th'+sc+'_'+datatype+'_dsl'
;    or whatever is specified by the newname keyword.
; Restrictions:
;      Required data must be loaded, including spin phase data.
; Notes:
;    This routine ignores E56 when despinning.  That is desireable over
;      the full transform (a la THM_COTRANS.PRO) because the resolution on E56
;      is much lower than for the spin plane sensors (E12 and E34) so mixing
;      in E56 would often increase the noise, and because the geometric
;      axis is w/in 1 degree of the spin axis making the result effectively
;      in DSL.
;
; Modifications:
;    Switched from THM_SPIN_PHASE.PRO to Jim Lewis' SPINMODEL_INTERP_T.PRO,
;      (commented out GET_DATA's to _state_spinper/spinphase, modified error
;      conditional), edited warning message, up'd doc'n, W.M.Feuerstein,
;      3/19/08.
;    Added TPLOT_NAME to specify var. to read.  Otherwise, PROBE and DATATYPE
;      parameters are used to derive tplot var. name to read.  Changed default
;      suffix to "_dsz" to match the "THEMIS Science Coordinate Systems
;      Definition" (THM-SOC-110).
;    Made sure "V" field is carried through data (otherwise it crashes
;      THM_CAL_EFI.PRO), redined suffix to "_dsl" (effectively true), fixed
;      potential bitwise vs. logical conditional bug, updated doc'n, WMF,
;      3/21/2008 (F).
;    Put in return on errors w/ messages for DLIMITS.DATA_ATT.COORD_SYS equal
;      to 'dsl' or 'spg', set said field to 'dsl' on conclusion of
;      processing, updat'd doc'n, WMF, 3/24/2008.
;    Checks validity of pointer from SPINMODEL_GET_PTR.PRO and calls
;      THM_LOAD_SPIN,PROBE=PROBE[0] on fail, installed an alternate test for
;      spinmodel data (TNAMES('th?_spin*')) to check an error from
;      SPINMODEL_INTERP_T.PRO but the error is unrelated, WMF, 3/26/2008.
;    Removed check on pointer in favor of incorporating THM_LOAD_SPIN.PRO into
;      THM_LOAD_STATE.PRO, WMF, 3/27/2008.
;    Updated doc'n, WMF, 3/27/2008.
;    Changed "Despinning <variable>" PRINT to MESSAGE, if data already despun
;      save TPLOT var. instead of just returning (BugzID 111), WMF, 4/4/08 (F).
;    Disabled BOOM_OFFSET parameter which is obsolete, WMF, 4/22/2008 (Tu).
;---------------------------------------------------------------
;pro thm_efi_despin,probe,datatype,offset,gain,boom_offset,notch=notch, $
;  newname=newname,tweak_gains=tweak_gains, tplot_name=tplot_name
pro thm_efi_despin,probe,datatype,offset,gain,notch=notch, $
  newname=newname,tweak_gains=tweak_gains, tplot_name=tplot_name


  ;==========
  ; Get data:
  ;==========
  if ~size(tplot_name,/type) then $
    get_data,'th'+probe[0]+'_'+datatype,data=data,lim=lim,dlim=dlim else $
    get_data,tplot_name,data=data,lim=lim,dlim=dlim
  eff_t=data.x
  eff=data.y
  ;

  ;===========================
  ;Check that data is present:
  ;===========================
  if n_elements(eff_t) eq 1 then begin
       print,'ERROR in thm_efi_despin: data not available.  Returning...'
       return
  endif


  ;=========================================================================
  ;Return if data is in a coord. sys. that this routine is not designed for:
  ;=========================================================================
  coord=cotrans_get_coord(dlim)
  if coord ne 'spg' && coord ne 'dsl' then begin
    message,'*** WARNING!:  THM_EFI_DESPIN.PRO only accepts input data in '+ $
      'SPG or DSL coordinates.  Returning...'
    return
  endif

  ;===================================
  ;If data is already in the DSL coordinate system,
  ;assign Ex,Ey,Ez without despinning:
  ;===================================
  if coord eq 'dsl' then begin
    message,'*** WARNING!:  Data is already despun.  '+ $
      'Saving without applying EFI despin...',/continue
    Ex=eff[*,0]
    Ey=eff[*,1]
    Ez=eff[*,2]


  ;=============
  ;Else, despin:
  ;=============
  endif else begin


    ;=================================================
    ;Notify the user that EFI despin is being applied:
    ;=================================================
    if ~(~size(tplot_name,/type)) then $
      message,'Applying EFI despin to: '+tplot_name,/cont else $
      message,'Applying EFI despin to: ','th'+probe[0]+'_'+datatype,/cont


    ;=======================
    ;Interpolate spin phase:
    ;=======================
    model=spinmodel_get_ptr(probe[0])
    spinmodel_interp_t,model=model,time=eff_t, $
      spinphase=phase,spinper=spinper              ;a la J. L.


    ;This section is obsolete.
    ;=========================
    ;;===================
    ;;Remove boom offset:
    ;;===================
    ;if n_elements(boom_offset) lt 2 then begin
    ;  eff[*,0]-=smooth(eff[*,0],500)
    ;  eff[*,1]-=smooth(eff[*,1],500)
    ;endif else begin
    ;  eff[*,0]-=boom_offset[0]
    ;  eff[*,1]-=boom_offset[1]
    ;endelse


    ;============================
    ;Despin and subtract offsets:
    ;============================
    phase*=!dtor
    phase-=45*!dtor
    if keyword_set(tweak_gains) then begin
      Ex1 = eff[*,0]*gain[0]*sin(phase)
      Ex2 = eff[*,1]*gain[1]*cos(phase)
      Ey1 = -eff[*,0]*gain[0]*cos(phase)
      Ey2 = eff[*,1]*gain[1]*sin(phase)
      tweak1 = smooth(Ex1,60)/smooth(Ex2,60)
      tweak2 = smooth(Ey1,60)/smooth(Ey2,60)
      Ex = Ex1+Ex2*median(tweak1)
      Ey = Ey1+Ey2*median(tweak1)
    endif else begin
      Ex = eff[*,0]*gain[0]*sin(phase)  + eff[*,1]*gain[1]*cos(phase)
      Ey = -eff[*,0]*gain[0]*cos(phase) + eff[*,1]*gain[1]*sin(phase)
    endelse
    Ez = eff[*,2]*gain[2]
    Ex-=offset[0]
    Ey-=offset[1]
    Ez-=offset[2]


    ;===========================
    ;Notch filter (if required):
    ;===========================
    if n_elements(notch) ne 0 then if notch gt 0 then begin
      rate = eff_t[1]-eff_t[0]
      notch_filter,Ex,notch,rate
      notch_filter,Ey,notch,rate
      notch_filter,Ez,notch,rate
    endif


    ;======================
    ;Change COORD_SYS flag:
    ;======================
    cotrans_set_coord,dlim,'dsl'

  endelse


  ;===========
  ;Write data:
  ;===========
  if ~(~size(tplot_name,/type)) then begin
    if n_elements(newname) eq 0 then newname=tplot_name+'_dsl'
  endif else begin
    if n_elements(newname) eq 0 then newname='th'+probe[0]+'_'+datatype+'_dsl'
  endelse
  store_data,newname,data={x:eff_t,y:[[Ex],[Ey],[Ez]],v:data.v}, $
    lim=lim,dlim=dlim
  print,'   Stored despun data (SPG -> DSL) in tplot variable ',newname

end
