;+
;PROCEDURE:	thm_load_esa_pot
;PURPOSE:	
;	Add spacecraft potential data to the ESA structures
;INPUT:		
;
;KEYWORDS:
;	probe:		string		themis spacecraft - "a", "b", "c", "d", "e"
;					if not set defaults to "a"
;	sc:		string		themis spacecraft - "a", "b", "c", "d", "e"
;					if not set defaults to "a"				
;	themishome:	string		path to data dir, where data dir contains the th* dir, where *=a,b,c,d,e
;	datatype	string or 0/1	if not set, uses a combination of spin averaged moment data and v1234 spin averaged data
;					if set to 1, uses vaf v12 data
;					if string, uses string data type - mom or v12, v34, v1234 where the latter are vaf data
;					vaf data is averaged by "time_average" with resolution=3s before interpolation 
;					s/c potential data are interpolated to center of esa data collection
;	pot_scale:	real		scale factor applied to measured EFI sensor potential, default=1.15
;	offset:		real		EFI sensor to plasma potential offset, default=1.0 V
;	min_pot:	real		minimum potential allowed, default=2.0
;	make_plot:	0/1		if set, makes tplot structure of potential 
;	trange:		str/dbl		2 element array of times to fix s/c pot at min_pot
;
;CREATED BY:	J. McFadden	  07-07-23
;VERSION:	1
;LAST MODIFICATION:  08-01-02
;MOD HISTORY:
;			07-12-27	corrected mom packet timing and potential
;			08-01-02	added trange keyword
;			08-04-16	default datatype changed to optimize between mom-pot and vaf-v1234-pot
;NOTES:	  
;	
;-

pro thm_load_esa_pot,sc=sc,probe=probe,themishome=themishome,datatype=datatype,pot_scale=pot_scale,offset=offset,min_pot=min_pot,make_plot=make_plot,trange=trange

; moment packet potential must be time shifted, onboard timing changed at mom_tim_adjust
; 1.6028 = 1 + 217/360 times spin_period is the spin offset time between s/c pot in moments packet and actual time s/c potential calculated
; after mom_tim_adjust[], 0.625 times spin_period is the spin offset time between s/c pot in moments packet and actual time s/c potential calculated
; changes to timing for s/c potential in moments packets occurred at
; 	THEMIS A: 07-333-20:51:26
; 	THEMIS B: 07-337-18:43:24
; 	THEMIS C: 07-337-18:23:03
; 	THEMIS D: 07-331-18:34:23
; 	THEMIS E: 07-333-17:49:10
;
; moment packet potential must be rescaled after below dates (mom_pot_adjust) to compensate for onboard scaling
; ICCR_MML28 - Adjust spacecraft potential scale & offset for pot in moments packets
;	THEMIS A 07-321-19:03:49
;	THEMIS B 07-322-03:12:17
;	THEMIS C 07-322-07:12:01
;	THEMIS D 07-322-00:02:15
;	THEMIS E 07-322-01:55:32
;

mom_pot_adjust=time_double(['07-11-17/19:03:49','07-11-18/03:12:17','07-11-18/07:12:01','07-11-18/00:02:15','07-11-18/01:55:32'])
mom_tim_adjust=time_double(['07-11-29/20:51:26','07-12-03/18:43:24','07-12-03/18:23:03','07-11-27/18:34:23','07-11-29/17:49:10'])
tshft_mom=[1.6028,0.625]

; sc default
	if keyword_set(probe) then sc=probe
	if not keyword_set(sc) then begin
		print,'S/C number not set, default = all probes'
		sc=['a','b','c','d','e']
	endif

	if not keyword_set(themishome) then themishome=!themis.local_data_dir

nsc = n_elements(sc)
probes=strarr(1)
if nsc eq 1 then probes[0]=sc
if nsc ne 1 then probes=sc

if not keyword_set(pot_scale) then scale=1.15 else scale=pot_scale
if not keyword_set(offset) then offset=1.0
if not keyword_set(min_pot) then min_pot=2.0

;***********************************************************************************
; get s/c potential data

if keyword_set(trange) then begin
	if n_elements(trange) ne 2 then begin
		print,'trange keyword must be 2 element array'
		return
	endif
	tr2=time_double(trange)
	if tr2[0] gt tr2[1] then tr2=reverse(tr2)
	
  for i=0,nsc-1 do begin
	if probes[i] eq 'a' then begin
		common tha_454,tha_454_ind,tha_454_dat 
		if n_elements(tha_454_dat) ne 0 then begin
			ind=where(tha_454_dat.time ge tr2[0] and tha_454_dat.time le tr2[1])
			if ind[0] ne -1 then tha_454_dat.sc_pot[ind]=min_pot
			store_data,'tha_peif_sc_pot',data={x:(tha_454_dat.time+tha_454_dat.end_time)/2.,y:tha_454_dat.sc_pot}
		endif
		common tha_455,tha_455_ind,tha_455_dat 
		if n_elements(tha_455_dat) ne 0 then begin
			ind=where(tha_455_dat.time ge tr2[0] and tha_455_dat.time le tr2[1])
			if ind[0] ne -1 then tha_455_dat.sc_pot[ind]=min_pot
			store_data,'tha_peir_sc_pot',data={x:(tha_455_dat.time+tha_455_dat.end_time)/2.,y:tha_455_dat.sc_pot}
		endif
		common tha_456,tha_456_ind,tha_456_dat 
		if n_elements(tha_456_dat) ne 0 then begin
			ind=where(tha_456_dat.time ge tr2[0] and tha_456_dat.time le tr2[1])
			if ind[0] ne -1 then tha_456_dat.sc_pot[ind]=min_pot
			store_data,'tha_peib_sc_pot',data={x:(tha_456_dat.time+tha_456_dat.end_time)/2.,y:tha_456_dat.sc_pot}
		endif
		common tha_457,tha_457_ind,tha_457_dat 
		if n_elements(tha_457_dat) ne 0 then begin
			ind=where(tha_457_dat.time ge tr2[0] and tha_457_dat.time le tr2[1])
			if ind[0] ne -1 then tha_457_dat.sc_pot[ind]=min_pot
			store_data,'tha_peef_sc_pot',data={x:(tha_457_dat.time+tha_457_dat.end_time)/2.,y:tha_457_dat.sc_pot}
		endif
		common tha_458,tha_458_ind,tha_458_dat 
		if n_elements(tha_458_dat) ne 0 then begin
			ind=where(tha_458_dat.time ge tr2[0] and tha_458_dat.time le tr2[1])
			if ind[0] ne -1 then tha_458_dat.sc_pot[ind]=min_pot
			store_data,'tha_peer_sc_pot',data={x:(tha_458_dat.time+tha_458_dat.end_time)/2.,y:tha_458_dat.sc_pot}
		endif
		common tha_459,tha_459_ind,tha_459_dat 
		if n_elements(tha_459_dat) ne 0 then begin
			ind=where(tha_459_dat.time ge tr2[0] and tha_459_dat.time le tr2[1])
			if ind[0] ne -1 then tha_459_dat.sc_pot[ind]=min_pot
			store_data,'tha_peeb_sc_pot',data={x:(tha_459_dat.time+tha_459_dat.end_time)/2.,y:tha_459_dat.sc_pot}
		endif
	endif else if probes[i] eq 'b' then begin
		common thb_454,thb_454_ind,thb_454_dat 
		if n_elements(thb_454_dat) ne 0 then begin
			ind=where(thb_454_dat.time ge tr2[0] and thb_454_dat.time le tr2[1])
			if ind[0] ne -1 then thb_454_dat.sc_pot[ind]=min_pot
			store_data,'thb_peif_sc_pot',data={x:(thb_454_dat.time+thb_454_dat.end_time)/2.,y:thb_454_dat.sc_pot}
		endif
		common thb_455,thb_455_ind,thb_455_dat 
		if n_elements(thb_455_dat) ne 0 then begin
			ind=where(thb_455_dat.time ge tr2[0] and thb_455_dat.time le tr2[1])
			if ind[0] ne -1 then thb_455_dat.sc_pot[ind]=min_pot
			store_data,'thb_peir_sc_pot',data={x:(thb_455_dat.time+thb_455_dat.end_time)/2.,y:thb_455_dat.sc_pot}
		endif
		common thb_456,thb_456_ind,thb_456_dat 
		if n_elements(thb_456_dat) ne 0 then begin
			ind=where(thb_456_dat.time ge tr2[0] and thb_456_dat.time le tr2[1])
			if ind[0] ne -1 then thb_456_dat.sc_pot[ind]=min_pot
			store_data,'thb_peib_sc_pot',data={x:(thb_456_dat.time+thb_456_dat.end_time)/2.,y:thb_456_dat.sc_pot}
		endif
		common thb_457,thb_457_ind,thb_457_dat 
		if n_elements(thb_457_dat) ne 0 then begin
			ind=where(thb_457_dat.time ge tr2[0] and thb_457_dat.time le tr2[1])
			if ind[0] ne -1 then thb_457_dat.sc_pot[ind]=min_pot
			store_data,'thb_peef_sc_pot',data={x:(thb_457_dat.time+thb_457_dat.end_time)/2.,y:thb_457_dat.sc_pot}
		endif
		common thb_458,thb_458_ind,thb_458_dat 
		if n_elements(thb_458_dat) ne 0 then begin
			ind=where(thb_458_dat.time ge tr2[0] and thb_458_dat.time le tr2[1])
			if ind[0] ne -1 then thb_458_dat.sc_pot[ind]=min_pot
			store_data,'thb_peer_sc_pot',data={x:(thb_458_dat.time+thb_458_dat.end_time)/2.,y:thb_458_dat.sc_pot}
		endif
		common thb_459,thb_459_ind,thb_459_dat 
		if n_elements(thb_459_dat) ne 0 then begin
			ind=where(thb_459_dat.time ge tr2[0] and thb_459_dat.time le tr2[1])
			if ind[0] ne -1 then thb_459_dat.sc_pot[ind]=min_pot
			store_data,'thb_peeb_sc_pot',data={x:(thb_459_dat.time+thb_459_dat.end_time)/2.,y:thb_459_dat.sc_pot}
		endif
	endif else if probes[i] eq 'c' then begin
		common thc_454,thc_454_ind,thc_454_dat 
		if n_elements(thc_454_dat) ne 0 then begin
			ind=where(thc_454_dat.time ge tr2[0] and thc_454_dat.time le tr2[1])
			if ind[0] ne -1 then thc_454_dat.sc_pot[ind]=min_pot
			store_data,'thc_peif_sc_pot',data={x:(thc_454_dat.time+thc_454_dat.end_time)/2.,y:thc_454_dat.sc_pot}
		endif
		common thc_455,thc_455_ind,thc_455_dat 
		if n_elements(thc_455_dat) ne 0 then begin
			ind=where(thc_455_dat.time ge tr2[0] and thc_455_dat.time le tr2[1])
			if ind[0] ne -1 then thc_455_dat.sc_pot[ind]=min_pot
			store_data,'thc_peir_sc_pot',data={x:(thc_455_dat.time+thc_455_dat.end_time)/2.,y:thc_455_dat.sc_pot}
		endif
		common thc_456,thc_456_ind,thc_456_dat 
		if n_elements(thc_456_dat) ne 0 then begin
			ind=where(thc_456_dat.time ge tr2[0] and thc_456_dat.time le tr2[1])
			if ind[0] ne -1 then thc_456_dat.sc_pot[ind]=min_pot
			store_data,'thc_peib_sc_pot',data={x:(thc_456_dat.time+thc_456_dat.end_time)/2.,y:thc_456_dat.sc_pot}
		endif
		common thc_457,thc_457_ind,thc_457_dat 
		if n_elements(thc_457_dat) ne 0 then begin
			ind=where(thc_457_dat.time ge tr2[0] and thc_457_dat.time le tr2[1])
			if ind[0] ne -1 then thc_457_dat.sc_pot[ind]=min_pot
			store_data,'thc_peef_sc_pot',data={x:(thc_457_dat.time+thc_457_dat.end_time)/2.,y:thc_457_dat.sc_pot}
		endif
		common thc_458,thc_458_ind,thc_458_dat 
		if n_elements(thc_458_dat) ne 0 then begin
			ind=where(thc_458_dat.time ge tr2[0] and thc_458_dat.time le tr2[1])
			if ind[0] ne -1 then thc_458_dat.sc_pot[ind]=min_pot
			store_data,'thc_peer_sc_pot',data={x:(thc_458_dat.time+thc_458_dat.end_time)/2.,y:thc_458_dat.sc_pot}
		endif
		common thc_459,thc_459_ind,thc_459_dat 
		if n_elements(thc_459_dat) ne 0 then begin
			ind=where(thc_459_dat.time ge tr2[0] and thc_459_dat.time le tr2[1])
			if ind[0] ne -1 then thc_459_dat.sc_pot[ind]=min_pot
			store_data,'thc_peeb_sc_pot',data={x:(thc_459_dat.time+thc_459_dat.end_time)/2.,y:thc_459_dat.sc_pot}
		endif
	endif else if probes[i] eq 'd' then begin
		common thd_454,thd_454_ind,thd_454_dat 
		if n_elements(thd_454_dat) ne 0 then begin
			ind=where(thd_454_dat.time ge tr2[0] and thd_454_dat.time le tr2[1])
			if ind[0] ne -1 then thd_454_dat.sc_pot[ind]=min_pot
			store_data,'thd_peif_sc_pot',data={x:(thd_454_dat.time+thd_454_dat.end_time)/2.,y:thd_454_dat.sc_pot}
		endif
		common thd_455,thd_455_ind,thd_455_dat 
		if n_elements(thd_455_dat) ne 0 then begin
			ind=where(thd_455_dat.time ge tr2[0] and thd_455_dat.time le tr2[1])
			if ind[0] ne -1 then thd_455_dat.sc_pot[ind]=min_pot
			store_data,'thd_peir_sc_pot',data={x:(thd_455_dat.time+thd_455_dat.end_time)/2.,y:thd_455_dat.sc_pot}
		endif
		common thd_456,thd_456_ind,thd_456_dat 
		if n_elements(thd_456_dat) ne 0 then begin
			ind=where(thd_456_dat.time ge tr2[0] and thd_456_dat.time le tr2[1])
			if ind[0] ne -1 then thd_456_dat.sc_pot[ind]=min_pot
			store_data,'thd_peib_sc_pot',data={x:(thd_456_dat.time+thd_456_dat.end_time)/2.,y:thd_456_dat.sc_pot}
		endif
		common thd_457,thd_457_ind,thd_457_dat 
		if n_elements(thd_457_dat) ne 0 then begin
			ind=where(thd_457_dat.time ge tr2[0] and thd_457_dat.time le tr2[1])
			if ind[0] ne -1 then thd_457_dat.sc_pot[ind]=min_pot
			store_data,'thd_peef_sc_pot',data={x:(thd_457_dat.time+thd_457_dat.end_time)/2.,y:thd_457_dat.sc_pot}
		endif
		common thd_458,thd_458_ind,thd_458_dat 
		if n_elements(thd_458_dat) ne 0 then begin
			ind=where(thd_458_dat.time ge tr2[0] and thd_458_dat.time le tr2[1])
			if ind[0] ne -1 then thd_458_dat.sc_pot[ind]=min_pot
			store_data,'thd_peer_sc_pot',data={x:(thd_458_dat.time+thd_458_dat.end_time)/2.,y:thd_458_dat.sc_pot}
		endif
		common thd_459,thd_459_ind,thd_459_dat 
		if n_elements(thd_459_dat) ne 0 then begin
			ind=where(thd_459_dat.time ge tr2[0] and thd_459_dat.time le tr2[1])
			if ind[0] ne -1 then thd_459_dat.sc_pot[ind]=min_pot
			store_data,'thd_peeb_sc_pot',data={x:(thd_459_dat.time+thd_459_dat.end_time)/2.,y:thd_459_dat.sc_pot}
		endif
	endif else if probes[i] eq 'e' then begin
		common the_454,the_454_ind,the_454_dat 
		if n_elements(the_454_dat) ne 0 then begin
			ind=where(the_454_dat.time ge tr2[0] and the_454_dat.time le tr2[1])
			if ind[0] ne -1 then the_454_dat.sc_pot[ind]=min_pot
			store_data,'the_peif_sc_pot',data={x:(the_454_dat.time+the_454_dat.end_time)/2.,y:the_454_dat.sc_pot}
		endif
		common the_455,the_455_ind,the_455_dat 
		if n_elements(the_455_dat) ne 0 then begin
			ind=where(the_455_dat.time ge tr2[0] and the_455_dat.time le tr2[1])
			if ind[0] ne -1 then the_455_dat.sc_pot[ind]=min_pot
			store_data,'the_peir_sc_pot',data={x:(the_455_dat.time+the_455_dat.end_time)/2.,y:the_455_dat.sc_pot}
		endif
		common the_456,the_456_ind,the_456_dat 
		if n_elements(the_456_dat) ne 0 then begin
			ind=where(the_456_dat.time ge tr2[0] and the_456_dat.time le tr2[1])
			if ind[0] ne -1 then the_456_dat.sc_pot[ind]=min_pot
			store_data,'the_peib_sc_pot',data={x:(the_456_dat.time+the_456_dat.end_time)/2.,y:the_456_dat.sc_pot}
		endif
		common the_457,the_457_ind,the_457_dat 
		if n_elements(the_457_dat) ne 0 then begin
			ind=where(the_457_dat.time ge tr2[0] and the_457_dat.time le tr2[1])
			if ind[0] ne -1 then the_457_dat.sc_pot[ind]=min_pot
			store_data,'the_peef_sc_pot',data={x:(the_457_dat.time+the_457_dat.end_time)/2.,y:the_457_dat.sc_pot}
		endif
		common the_458,the_458_ind,the_458_dat 
		if n_elements(the_458_dat) ne 0 then begin
			ind=where(the_458_dat.time ge tr2[0] and the_458_dat.time le tr2[1])
			if ind[0] ne -1 then the_458_dat.sc_pot[ind]=min_pot
			store_data,'the_peer_sc_pot',data={x:(the_458_dat.time+the_458_dat.end_time)/2.,y:the_458_dat.sc_pot}
		endif
		common the_459,the_459_ind,the_459_dat 
		if n_elements(the_459_dat) ne 0 then begin
			ind=where(the_459_dat.time ge tr2[0] and the_459_dat.time le tr2[1])
			if ind[0] ne -1 then the_459_dat.sc_pot[ind]=min_pot
			store_data,'the_peeb_sc_pot',data={x:(the_459_dat.time+the_459_dat.end_time)/2.,y:the_459_dat.sc_pot}
		endif
	endif else if probes[i] eq 'f' then begin
		common thf_454,thf_454_ind,thf_454_dat 
		if n_elements(thf_454_dat) ne 0 then begin
			ind=where(thf_454_dat.time ge tr2[0] and thf_454_dat.time le tr2[1])
			if ind[0] ne -1 then thf_454_dat.sc_pot[ind]=min_pot
			store_data,'thf_peif_sc_pot',data={x:(thf_454_dat.time+thf_454_dat.end_time)/2.,y:thf_454_dat.sc_pot}
		endif
		common thf_455,thf_455_ind,thf_455_dat 
		if n_elements(thf_455_dat) ne 0 then begin
			ind=where(thf_455_dat.time ge tr2[0] and thf_455_dat.time le tr2[1])
			if ind[0] ne -1 then thf_455_dat.sc_pot[ind]=min_pot
			store_data,'thf_peir_sc_pot',data={x:(thf_455_dat.time+thf_455_dat.end_time)/2.,y:thf_455_dat.sc_pot}
		endif
		common thf_456,thf_456_ind,thf_456_dat 
		if n_elements(thf_456_dat) ne 0 then begin
			ind=where(thf_456_dat.time ge tr2[0] and thf_456_dat.time le tr2[1])
			if ind[0] ne -1 then thf_456_dat.sc_pot[ind]=min_pot
			store_data,'thf_peib_sc_pot',data={x:(thf_456_dat.time+thf_456_dat.end_time)/2.,y:thf_456_dat.sc_pot}
		endif
		common thf_457,thf_457_ind,thf_457_dat 
		if n_elements(thf_457_dat) ne 0 then begin
			ind=where(thf_457_dat.time ge tr2[0] and thf_457_dat.time le tr2[1])
			if ind[0] ne -1 then thf_457_dat.sc_pot[ind]=min_pot
			store_data,'thf_peef_sc_pot',data={x:(thf_457_dat.time+thf_457_dat.end_time)/2.,y:thf_457_dat.sc_pot}
		endif
		common thf_458,thf_458_ind,thf_458_dat 
		if n_elements(thf_458_dat) ne 0 then begin
			ind=where(thf_458_dat.time ge tr2[0] and thf_458_dat.time le tr2[1])
			if ind[0] ne -1 then thf_458_dat.sc_pot[ind]=min_pot
			store_data,'thf_peer_sc_pot',data={x:(thf_458_dat.time+thf_458_dat.end_time)/2.,y:thf_458_dat.sc_pot}
		endif
		common thf_459,thf_459_ind,thf_459_dat 
		if n_elements(thf_459_dat) ne 0 then begin
			ind=where(thf_459_dat.time ge tr2[0] and thf_459_dat.time le tr2[1])
			if ind[0] ne -1 then thf_459_dat.sc_pot[ind]=min_pot
			store_data,'thf_peeb_sc_pot',data={x:(thf_459_dat.time+thf_459_dat.end_time)/2.,y:thf_459_dat.sc_pot}
		endif
	endif

  endfor

endif else begin

  for i=0,nsc-1 do begin

	if probes[i] ne 'f' then begin
	if not keyword_set(datatype) then begin
		thm_load_mom,probe=probes[i]
		get_data,'th'+probes[i]+'_pxxm_pot',data=tmp
			npts=n_elements(tmp.x)
			if tmp.x[npts-1] gt mom_pot_adjust[i] then begin
				if tmp.x[0] ge mom_pot_adjust[i] then tmp.y=(tmp.y-1.)/1.15 else begin
				min_tim=min(abs(tmp.x-mom_pot_adjust[i]),ind)
				if tmp.x[ind] gt mom_pot_adjust[i] then ind=ind-1
				tmp.y[ind+1:npts-1]=(tmp.y[ind+1:npts-1]-1.)/1.15
				endelse
			endif

		get_data,'th'+probes[i]+'_state_spinper',data=spinper
		if not keyword_set(spinper) then begin
			thm_load_state,/get_support_data,probe=probes[i]
			get_data,'th'+probes[i]+'_state_spinper',data=spinper
			if not keyword_set(spinper) then begin
				print,'No state data available for probe ',probes[i]
				print,'ESA s/c potential load aborted'
				return
			endif
		endif
		spin_period = interp(spinper.y,spinper.x,tmp.x)

		if tmp.x[0] ge mom_tim_adjust[i] then time=tmp.x-tshft_mom[1]*spin_period else $
		if tmp.x[npts-1] le mom_tim_adjust[i] then time=tmp.x-tshft_mom[0]*spin_period else begin
			min_tim=min(abs(tmp.x-mom_tim_adjust[i]),ind)
			if tmp.x[ind] gt mom_tim_adjust[i] then ind=ind-1
			time=tmp.x
			time[0:ind]=time[0:ind]-tshft_mom[0]*spin_period[0:ind]
			time[ind+1:npts-1]=time[ind+1:npts-1]-tshft_mom[1]*spin_period[ind+1:npts-1]
		endelse	
		scpot=tmp.y

		avg_spin_period=total(spin_period)/npts
		thm_load_efi,probe=probes[i],datatype='vaf',level=1
		get_data,'th'+probes[i]+'_vaf',data=tmp1
		vaf12 = -0.5*reform(tmp1.y[*,0]+tmp1.y[*,1])
		vaf34 = -0.5*reform(tmp1.y[*,2]+tmp1.y[*,3])
		ind2 = where(vaf12/vaf34 gt 1.5 and vaf34 gt 1.) 
		if ind2(0) ne -1 then vaf12[ind2]=vaf34[ind2]		; get rid of bad points
		ind2 = where(vaf34/vaf12 gt 1.5 and vaf12 gt 1.) 
		if ind2(0) ne -1 then vaf34[ind2]=vaf12[ind2]		; get rid of bad points
		vaf1234 = -.25*reform(tmp1.y[*,0]+tmp1.y[*,1]+tmp1.y[*,2]+tmp1.y[*,3])
		vaf1234_3s=time_average(tmp1.x,vaf1234,resolution=avg_spin_period,newtime=newtime)

		t3 = [time,newtime]
		d3 = [scpot,vaf1234_3s]
		s = sort(t3)
		time=t3[s]
		scpot=(scale*(d3[s]+offset)) > min_pot
		store_data,'th'+probes[i]+'_esa_pot',data={x:time,y:scpot}

	endif else if string(datatype) eq 'mom' then begin
		thm_load_mom,probe=probes[i]
		get_data,'th'+probes[i]+'_pxxm_pot',data=tmp
			npts=n_elements(tmp.x)
			if tmp.x[npts-1] gt mom_pot_adjust[i] then begin
				if tmp.x[0] ge mom_pot_adjust[i] then tmp.y=(tmp.y-1.)/1.15 else begin
				min_tim=min(abs(tmp.x-mom_pot_adjust[i]),ind)
				if tmp.x[ind] gt mom_pot_adjust[i] then ind=ind-1
				tmp.y[ind+1:npts-1]=(tmp.y[ind+1:npts-1]-1.)/1.15
				endelse
			endif

		get_data,'th'+probes[i]+'_state_spinper',data=spinper
		if not keyword_set(spinper) then begin
			thm_load_state,/get_support_data,probe=probes[i]
			get_data,'th'+probes[i]+'_state_spinper',data=spinper
			if not keyword_set(spinper) then begin
				print,'No state data available for probe ',probes[i]
				print,'ESA s/c potential load aborted'
				return
			endif
		endif
		spin_period = interp(spinper.y,spinper.x,tmp.x)

		if tmp.x[0] ge mom_tim_adjust[i] then time=tmp.x-tshft_mom[1]*spin_period else $
		if tmp.x[npts-1] le mom_tim_adjust[i] then time=tmp.x-tshft_mom[0]*spin_period else begin
			min_tim=min(abs(tmp.x-mom_tim_adjust[i]),ind)
			if tmp.x[ind] gt mom_tim_adjust[i] then ind=ind-1
			time=tmp.x
			time[0:ind]=time[0:ind]-tshft_mom[0]*spin_period[0:ind]
			time[ind+1:npts-1]=time[ind+1:npts-1]-tshft_mom[1]*spin_period[ind+1:npts-1]
		endelse	
		scpot=(scale*(tmp.y+offset)) > min_pot
		store_data,'th'+probes[i]+'_esa_pot',data={x:time,y:scpot}

	endif else begin
		thm_load_efi,probe=probes[i],datatype='vaf',level=1
		get_data,'th'+probes[i]+'_vaf',data=tmp1
		vaf12 = -0.5*reform(tmp1.y[*,0]+tmp1.y[*,1])
		vaf34 = -0.5*reform(tmp1.y[*,2]+tmp1.y[*,3])
		vaf1234 = -.25*reform(tmp1.y[*,0]+tmp1.y[*,1]+tmp1.y[*,2]+tmp1.y[*,3])
		vaf1234_3s=time_average(tmp1.x,vaf1234,resolution=3.,newtime=newtime)
		if keyword_set(make_plot) then store_data,'th'+probes[i]+'_v1234_3s',data={x:newtime,y:vaf1234_3s}
		vaf3s = vaf1234_3s
		vaf12_3s=time_average(tmp1.x,vaf12,resolution=3.,newtime=newtime)
		if keyword_set(make_plot) then store_data,'th'+probes[i]+'_v12_3s',data={x:newtime,y:vaf12_3s}
		if string(datatype) eq 'v12' then vaf3s = vaf12_3s
		vaf34_3s=time_average(tmp1.x,vaf34,resolution=3.,newtime=newtime)
		if keyword_set(make_plot) then store_data,'th'+probes[i]+'_v34_3s',data={x:newtime,y:vaf34_3s}
		if string(datatype) eq 'v34' then vaf3s = vaf34_3s
		if keyword_set(make_plot) then store_data,'th'+probes[i]+'_vaf_3s',data={x:newtime,y:vaf3s}
		time=newtime
		scpot=(scale*(vaf3s+offset)) > min_pot
		store_data,'th'+probes[i]+'_esa_pot',data={x:time,y:scpot}
		
	endelse
	endif

	if probes[i] eq 'a' then begin
		common tha_454,tha_454_ind,tha_454_dat 
		if n_elements(tha_454_dat) ne 0 then begin
		  if tha_454_ind ne -1 then begin
			tt=(tha_454_dat.time+tha_454_dat.end_time)/2. 
			tha_454_dat.sc_pot=interp(scpot,time,tt)
			store_data,'tha_peif_sc_pot',data={x:tt,y:tha_454_dat.sc_pot}
		  endif
		endif
		common tha_455,tha_455_ind,tha_455_dat 
		if n_elements(tha_455_dat) ne 0 then begin
		  if tha_455_ind ne -1 then begin
			tt=(tha_455_dat.time+tha_455_dat.end_time)/2.
			tha_455_dat.sc_pot=interp(scpot,time,tt)
			store_data,'tha_peir_sc_pot',data={x:tt,y:tha_455_dat.sc_pot}
		  endif
		endif
		common tha_456,tha_456_ind,tha_456_dat 
		if n_elements(tha_456_dat) ne 0 then begin
		  if tha_456_ind ne -1 then begin
			tt=(tha_456_dat.time+tha_456_dat.end_time)/2.
			tha_456_dat.sc_pot=interp(scpot,time,tt)
			store_data,'tha_peib_sc_pot',data={x:tt,y:tha_456_dat.sc_pot}
		  endif
		endif
		common tha_457,tha_457_ind,tha_457_dat 
		if n_elements(tha_457_dat) ne 0 then begin
		  if tha_457_ind ne -1 then begin
			tt=(tha_457_dat.time+tha_457_dat.end_time)/2.
			tha_457_dat.sc_pot=interp(scpot,time,tt)
			store_data,'tha_peef_sc_pot',data={x:tt,y:tha_457_dat.sc_pot}
		  endif
		endif
		common tha_458,tha_458_ind,tha_458_dat 
		if n_elements(tha_458_dat) ne 0 then begin
		  if tha_458_ind ne -1 then begin
			tt=(tha_458_dat.time+tha_458_dat.end_time)/2.
			tha_458_dat.sc_pot=interp(scpot,time,tt)
			store_data,'tha_peer_sc_pot',data={x:tt,y:tha_458_dat.sc_pot}
		  endif
		endif
		common tha_459,tha_459_ind,tha_459_dat 
		if n_elements(tha_459_dat) ne 0 then begin
		  if tha_459_ind ne -1 then begin
			tt=(tha_459_dat.time+tha_459_dat.end_time)/2.
			tha_459_dat.sc_pot=interp(scpot,time,tt)
			store_data,'tha_peeb_sc_pot',data={x:tt,y:tha_459_dat.sc_pot}
		  endif
		endif
	endif else if probes[i] eq 'b' then begin
		common thb_454,thb_454_ind,thb_454_dat 
		if n_elements(thb_454_dat) ne 0 then begin
		  if thb_454_ind ne -1 then begin
			tt=(thb_454_dat.time+thb_454_dat.end_time)/2.
			thb_454_dat.sc_pot=interp(scpot,time,tt)
			store_data,'thb_peif_sc_pot',data={x:tt,y:thb_454_dat.sc_pot}
		  endif
		endif
		common thb_455,thb_455_ind,thb_455_dat 
		if n_elements(thb_455_dat) ne 0 then begin
		  if thb_455_ind ne -1 then begin
			tt=(thb_455_dat.time+thb_455_dat.end_time)/2.
			thb_455_dat.sc_pot=interp(scpot,time,tt)
			store_data,'thb_peir_sc_pot',data={x:tt,y:thb_455_dat.sc_pot}
		  endif
		endif
		common thb_456,thb_456_ind,thb_456_dat 
		if n_elements(thb_456_dat) ne 0 then begin
		  if thb_456_ind ne -1 then begin
			tt=(thb_456_dat.time+thb_456_dat.end_time)/2.
			thb_456_dat.sc_pot=interp(scpot,time,tt)
			store_data,'thb_peib_sc_pot',data={x:tt,y:thb_456_dat.sc_pot}
		  endif
		endif
		common thb_457,thb_457_ind,thb_457_dat 
		if n_elements(thb_457_dat) ne 0 then begin
		  if thb_457_ind ne -1 then begin
			tt=(thb_457_dat.time+thb_457_dat.end_time)/2.
			thb_457_dat.sc_pot=interp(scpot,time,tt)
			store_data,'thb_peef_sc_pot',data={x:tt,y:thb_457_dat.sc_pot}
		  endif
		endif
		common thb_458,thb_458_ind,thb_458_dat 
		if n_elements(thb_458_dat) ne 0 then begin
		  if thb_458_ind ne -1 then begin
			tt=(thb_458_dat.time+thb_458_dat.end_time)/2.
			thb_458_dat.sc_pot=interp(scpot,time,tt)
			store_data,'thb_peer_sc_pot',data={x:tt,y:thb_458_dat.sc_pot}
		  endif
		endif
		common thb_459,thb_459_ind,thb_459_dat 
		if n_elements(thb_459_dat) ne 0 then begin
		  if thb_459_ind ne -1 then begin
			tt=(thb_459_dat.time+thb_459_dat.end_time)/2.
			thb_459_dat.sc_pot=interp(scpot,time,tt)
			store_data,'thb_peeb_sc_pot',data={x:tt,y:thb_459_dat.sc_pot}
		  endif
		endif
	endif else if probes[i] eq 'c' then begin
		common thc_454,thc_454_ind,thc_454_dat 
		if n_elements(thc_454_dat) ne 0 then begin
		  if thc_454_ind ne -1 then begin
			tt=(thc_454_dat.time+thc_454_dat.end_time)/2.
			thc_454_dat.sc_pot=interp(scpot,time,tt)
			store_data,'thc_peif_sc_pot',data={x:tt,y:thc_454_dat.sc_pot}
		  endif
		endif
		common thc_455,thc_455_ind,thc_455_dat 
		if n_elements(thc_455_dat) ne 0 then begin
		  if thc_455_ind ne -1 then begin
			tt=(thc_455_dat.time+thc_455_dat.end_time)/2.
			thc_455_dat.sc_pot=interp(scpot,time,tt)
			store_data,'thc_peir_sc_pot',data={x:tt,y:thc_455_dat.sc_pot}
		  endif
		endif
		common thc_456,thc_456_ind,thc_456_dat 
		if n_elements(thc_456_dat) ne 0 then begin
		  if thc_456_ind ne -1 then begin
			tt=(thc_456_dat.time+thc_456_dat.end_time)/2.
			thc_456_dat.sc_pot=interp(scpot,time,tt)
			store_data,'thc_peib_sc_pot',data={x:tt,y:thc_456_dat.sc_pot}
		  endif
		endif
		common thc_457,thc_457_ind,thc_457_dat 
		if n_elements(thc_457_dat) ne 0 then begin
		  if thc_457_ind ne -1 then begin
			tt=(thc_457_dat.time+thc_457_dat.end_time)/2.
			thc_457_dat.sc_pot=interp(scpot,time,tt)
			store_data,'thc_peef_sc_pot',data={x:tt,y:thc_457_dat.sc_pot}
		  endif
		endif
		common thc_458,thc_458_ind,thc_458_dat 
		if n_elements(thc_458_dat) ne 0 then begin
		  if thc_458_ind ne -1 then begin
			tt=(thc_458_dat.time+thc_458_dat.end_time)/2.
			thc_458_dat.sc_pot=interp(scpot,time,tt)
			store_data,'thc_peer_sc_pot',data={x:tt,y:thc_458_dat.sc_pot}
		  endif
		endif
		common thc_459,thc_459_ind,thc_459_dat 
		if n_elements(thc_459_dat) ne 0 then begin
		  if thc_459_ind ne -1 then begin
			tt=(thc_459_dat.time+thc_459_dat.end_time)/2.
			thc_459_dat.sc_pot=interp(scpot,time,tt)
			store_data,'thc_peeb_sc_pot',data={x:tt,y:thc_459_dat.sc_pot}
		  endif
		endif
	endif else if probes[i] eq 'd' then begin
		common thd_454,thd_454_ind,thd_454_dat 
		if n_elements(thd_454_dat) ne 0 then begin
		  if thd_454_ind ne -1 then begin
			tt=(thd_454_dat.time+thd_454_dat.end_time)/2.
			thd_454_dat.sc_pot=interp(scpot,time,tt)
			store_data,'thd_peif_sc_pot',data={x:tt,y:thd_454_dat.sc_pot}
		  endif
		endif
		common thd_455,thd_455_ind,thd_455_dat 
		if n_elements(thd_455_dat) ne 0 then begin
		  if thd_455_ind ne -1 then begin
			tt=(thd_455_dat.time+thd_455_dat.end_time)/2.
			thd_455_dat.sc_pot=interp(scpot,time,tt)
			store_data,'thd_peir_sc_pot',data={x:tt,y:thd_455_dat.sc_pot}
		  endif
		endif
		common thd_456,thd_456_ind,thd_456_dat 
		if n_elements(thd_456_dat) ne 0 then begin
		  if thd_456_ind ne -1 then begin
			tt=(thd_456_dat.time+thd_456_dat.end_time)/2.
			thd_456_dat.sc_pot=interp(scpot,time,tt)
			store_data,'thd_peib_sc_pot',data={x:tt,y:thd_456_dat.sc_pot}
		  endif
		endif
		common thd_457,thd_457_ind,thd_457_dat 
		if n_elements(thd_457_dat) ne 0 then begin
		  if thd_457_ind ne -1 then begin
			tt=(thd_457_dat.time+thd_457_dat.end_time)/2.
			thd_457_dat.sc_pot=interp(scpot,time,tt)
			store_data,'thd_peef_sc_pot',data={x:tt,y:thd_457_dat.sc_pot}
		  endif
		endif
		common thd_458,thd_458_ind,thd_458_dat 
		if n_elements(thd_458_dat) ne 0 then begin
		  if thd_458_ind ne -1 then begin
			tt=(thd_458_dat.time+thd_458_dat.end_time)/2.
			thd_458_dat.sc_pot=interp(scpot,time,tt)
			store_data,'thd_peer_sc_pot',data={x:tt,y:thd_458_dat.sc_pot}
		  endif
		endif
		common thd_459,thd_459_ind,thd_459_dat 
		if n_elements(thd_459_dat) ne 0 then begin
		  if thd_459_ind ne -1 then begin
			tt=(thd_459_dat.time+thd_459_dat.end_time)/2.
			thd_459_dat.sc_pot=interp(scpot,time,tt)
			store_data,'thd_peeb_sc_pot',data={x:tt,y:thd_459_dat.sc_pot}
		  endif
		endif
	endif else if probes[i] eq 'e' then begin
		common the_454,the_454_ind,the_454_dat 
		if n_elements(the_454_dat) ne 0 then begin
		  if the_454_ind ne -1 then begin
			tt=(the_454_dat.time+the_454_dat.end_time)/2.
			the_454_dat.sc_pot=interp(scpot,time,tt)
			store_data,'the_peif_sc_pot',data={x:tt,y:the_454_dat.sc_pot}
		  endif
		endif
		common the_455,the_455_ind,the_455_dat 
		if n_elements(the_455_dat) ne 0 then begin
		  if the_455_ind ne -1 then begin
			tt=(the_455_dat.time+the_455_dat.end_time)/2.
			the_455_dat.sc_pot=interp(scpot,time,tt)
			store_data,'the_peir_sc_pot',data={x:tt,y:the_455_dat.sc_pot}
		  endif
		endif
		common the_456,the_456_ind,the_456_dat 
		if n_elements(the_456_dat) ne 0 then begin
		  if the_456_ind ne -1 then begin
			tt=(the_456_dat.time+the_456_dat.end_time)/2.
			the_456_dat.sc_pot=interp(scpot,time,tt)
			store_data,'the_peib_sc_pot',data={x:tt,y:the_456_dat.sc_pot}
		  endif
		endif
		common the_457,the_457_ind,the_457_dat 
		if n_elements(the_457_dat) ne 0 then begin
		  if the_457_ind ne -1 then begin
			tt=(the_457_dat.time+the_457_dat.end_time)/2.
			the_457_dat.sc_pot=interp(scpot,time,tt)
			store_data,'the_peef_sc_pot',data={x:tt,y:the_457_dat.sc_pot}
		  endif
		endif
		common the_458,the_458_ind,the_458_dat 
		if n_elements(the_458_dat) ne 0 then begin
		  if the_458_ind ne -1 then begin
			tt=(the_458_dat.time+the_458_dat.end_time)/2.
			the_458_dat.sc_pot=interp(scpot,time,tt)
			store_data,'the_peer_sc_pot',data={x:tt,y:the_458_dat.sc_pot}
		  endif
		endif
		common the_459,the_459_ind,the_459_dat 
		if n_elements(the_459_dat) ne 0 then begin
		  if the_459_ind ne -1 then begin
			tt=(the_459_dat.time+the_459_dat.end_time)/2.
			the_459_dat.sc_pot=interp(scpot,time,tt)
			store_data,'the_peeb_sc_pot',data={x:tt,y:the_459_dat.sc_pot}
		  endif
		endif
	endif

  endfor
endelse

end
