;+
;PROCEDURE: thm_part_getspec
;PURPOSE:
;	Generates tplot variables containing energy-time angular particle spectra
;	for any angle range and/or any energy range.
;
;KEYWORDS:
; probe  = Probe name. The default is 'all', i.e., load all available probes.
;          This can be an array of strings, e.g., ['a', 'b'] or a
;          single string delimited by spaces, e.g., 'a b'
; trange = (Optional) Time range of interest  (2 element array), if
;          this is not set, the default is to prompt the user. Note
;          that if the input time range is not a full day, a full
;          day's data is loaded.
; phi    = Angle range of interest (2 element array) in degrees relative to
;          probe-sun direction in the probe's spin plane. Specify angles in
;          ascending order (e.g. [270, 450]) to specify the 'daylight'
;          hemisphere in DSL coordinates. Default is all 360 degrees (e.g.
;          [0, 360]). If the phi range is greater than 360 degrees, then the phi
;          bins at the beginning of the phi range are added and wrapped around
;          the end of phi range. For example, if phi=[0,420], the phi bins
;          corresponding to 0-60 degrees are appended to the top of the plot.
;          Also controls which bins are selected for pitch/gyrophase spectra.
; theta  = Angle range of interest (2 element array) in degrees relative to
;          spin plane, e.g. [-90, 0] or [-45, 45] in the probe's spin plane.
;          Specify in acending order. Default is all (e.g. [-90, 90]). Also
;          controls which bins are selected for pitch/gyrophase spectra.
; pitch  = Angle range of interest (2 element array) in degrees relative to
;          the magnetic field. Default is all (e.g. [0, 180]).
; gyro   = Gyrophase angle range of interest (2 element array) in degrees.
;          Same rules as specifying phi apply to gyrophse. Default is all
;          (e.g. [0, 360]). Has no effect on phi/theta/energy spectra plots.
; erange = Energy range (in eV) of interest (2 element array). Default is all.
; data_type = The type of data to be loaded (e.g. ['peif', 'peef']
; start_angle = Angle at which to begin plotting the data the bottom of the
;               plot along the y-axis. At this time this works for phi only. If
;               not set, keyword defaults to first element of phi input.
; suffix = Suffix to add to tplot variable names.
; /AUTOPLOT: Set to activiate a simple plotting routine to display tplot
;            variables created by this routine.
; /ENERGY: Set to output tplot variables containing the energy spectrum of the
;          energy/angle ranges input above.
; angle  = Type of angular spectrum tplot variable created and plotted using the
;          energy/angle ranges input above e.g. 'phi'. Default is 'phi'. The
;          type of angular spectra is appended to its corresponding tplot
;          variable name. All possible options are:
;          phi   = phi angular spectrum
;          theta = theta angular spectrum
;          pa    = pitch angular spectrum (only works with full angle ranges)
;          gyro  = gyrophase angular spectrum  (only works with full angle ranges)
;          NOTE: If neither of ENERGY and ANGLE keywords are specified, then
;                then both are turned on with ANGLE defaulting to 'phi'.
; /GET_SUPPORT_DATA: load support_data variables as well as data variables into
;                    tplot variables.
; regrid = [m,n]  Number of angle bins, used to re-grid (2-element array) along
;          phi (gyrophase) and theta (pitch), respectively, when calculating
;          pitch/gyrophase angle spectrum. For example, e.g. [32,16] creates a
;          regularly-spaced array of 32 phi angles by 16 theta angles that
;          resample the probe(s) native angle bin array. Default is [16,8].
;          Keyword has no effect on phi and theta spectragrams. Accuracy of the
;          spectragrams increases when more angle bins are used to re-grid.
;          Suitable numbers for m and n are numbers like 2^k. So far, k=2-6 has
;          been tested and work. Other numbers will work provided 180/n and
;          360/m are rational.
; other_dim = Keyword passed to THM_FAC_MATRIX_MAKE for conversion to field
;             aligned coordinates to create pitch angle and gyrophase spectra.
;             See THM_FAC_MATRIX_MAKE for valid input. Default is 'xgse'.
; /NORMALIZE: Set to normalize the flux for each time sample to 0-1.
; /MASK:   Set to enable masking of sun pulse in SST data. Automatically enabled
;          whenever BINS2MASK is called.
; bins2mask = [bin#, bin#2, ...] Array SST angle bin numbers that will be masked
;             with a NaN to eliminate sun contamination.
;
;EXAMPLE: thm_part_getspec, probe='d', trange=['07-06-17','07-06-19']
;
;SEE ALSO:
;	THM_CRIB_PART_GETSPEC, THM_PART_MOMENTS2, THM_PART_GETANBINS, THM_LOAD_SST,
;   THM_LOAD_ESA_PKT, THM_FAC_MATRIX_MAKE
;
;CREATED BY:	Bryan Kerr
;HISTORY:
; v0.1	11/20/07: Initial release.
; v0.2	11/28/07: Added ability to handle eESA and *SST data types
; v0.3  12/04/07: Added st_phi, tplotsuffix and autoplot keywords
; v0.4  12/11/07: Added ability to plot energy spectra,angle/energy keywords and
;                 angle/energy tplot variable setup routines. Also added wrapphi
;                 ability.
; v0.4.1 12/13/07:Set ANGLE and ENERGY keywords to both turn on if neither is
;                 specified. Moved start/end_angle module to THM_PART_MOMENTS2.
; v0.5  01/04/08: Changed tplotsuffix to suffix. Added NOMASK keyword. Reduced
;                 banding of masked SST data. ANGLE keyword now specifies type
;                 of angular spectrum.
; v0.6  01/09/08: Added reduced mode (peir) capability.
; v0.6.3 01/15/08: Implemented all reduced modes.
; v0.7  01/31/08: Added theta spectragram capability.
; v0.8  02/04/08: Alpha release of pitch angle spectragram capability.
; v0.8.01 02/06/08: Fixed bug that bombs thm_part_getspec if angle keyword not
;                   set.
; v0.8.1 02/20/08: Added /GET_SUPPORT_DATA keyword. Modified code to better
;                  handle 6-angle data.
; v0.8.11 02/20/08: Tplot title only created when /AUTOPLOT is specified.
; v0.8.12 02/26/08: Fixed bug causing fgm data to load if angle keyword not set.
; v0.8.5 03/26/08: Added REGRID keyword. Fixed bug that causes the code to bomb
;                  when a phi range is specified for theta spectragrams. Fixed
;                  bug preventing theta and pitch angle plots from showing full
;                  theta/pitch range requested by user. New algo to calc pitch
;                  angle and gyrophase spectra that's accurate and fast.
; v0.9 04/08/08: Added input checks for pitch/gyro keywords. Added ability to
;                specify phi, theta, pitch, gyrophase angle ranges for pitch and
;                gyrophase spectra. Added OTHER_DIM keyword for use by
;                THM_FAC_MATRIX make. Added en_tnames and an_tnames keywords to
;                make sure autoplot sets angle bounds only for newly created
;                tplot variables.
;
; v0.9.2 04/17/08: Fixed bug not allowing high regrid numbers (needed long int).
;                  Fixed bug producing all zeros in pitch/gyro plots when narrow
;                  phi/theta/energy ranges used. Added /NORMALIZE keyword.
;
; v0.9.3 04/18/08: Fixed bug that improperly handled pa/gyro SST data when
;                  rotated FAC bins fell into theta gaps of SST data between
;                  +/- 5 degrees. Fixed bug with /NORMALIZE keyword not
;                  properly handling NaNs for pitch and gyrophase plots.
; v0.9.31 04/21/08: Documentation modification for OTHER_DIM keyword.
; v0.9.32 04/21/08: Fixed call to THM_LOAD_STATE to account for mods in
;                   THM_FAC_MATRIX_MAKE.
; v0.9.4 04/25/08: Changed /NOMASK keyword to /MASK. Added BINS2MASK keyword.
; v0.9.41 04/25/08: Added _extra keyword so that /DEGAP and TDEGAP-related
;                   keywords could be passed to THM_FAC_MATRIX_MAKE.
; v1.0 05/09/08: Ready for Release v4.0.
;
;VERSION: 1.0
;  $LastChangedBy: bckerr $
;  $LastChangedDate: 2008-05-09 11:01:55 -0700 (Fri, 09 May 2008) $
;  $LastChangedRevision: 3056 $
;  $URL: svn+ssh://thmsvn@ambrosia.ssl.berkeley.edu/repos/thmsoc/tags/tdas_4_01/idl/themis/spacecraft/particles/thm_part_getspec.pro $
;-

pro thm_part_getspec, probe=probe, trange=trange, phi=phi, theta=theta, $
                      pitch=pitch, erange=erange, data_type=data_type, $
                      start_angle=start_angle, suffix=suffix, $
                      autoplot=autoplot, angle=angle, energy=energy, $
                      nomask=nomask, get_support_data=get_support_data, $
                      regrid=regrid, gyro=gyro, other_dim=other_dim, $
                      en_tnames=en_tnames, an_tnames=an_tnames, $
                      normalize=normalize, bins2mask=bins2mask, mask=mask, $
                      _extra=ex, test=test


if keyword_set(nomask) then begin
   print,''
   dprint, 'The /NOMASK keyword has been replaced by the /MASK keyword. ', $
           'Please see documentation (thm_crib_part_getspec) for usage info.'
   print,''
   return
endif

; Set default input
deftheta = [-90, 90]
defphi = [0, 360]
defpitch = [0, 180]
defgyro = defphi
deferange = [0, 1e7]
defdata_type = 'peif'
def_angle = 'phi'
defstart_angle = 0
wrapphi = 0

; Check for correct angle input
if keyword_set(theta) then begin
   if theta[1] lt theta[0] OR (theta[0] lt -90 OR theta[1] gt 90) then begin
      print,''
      dprint, 'Error with theta input. Does not satisfy: -90 < theta[0] < theta[1] < 90.'
      return
   endif
endif else theta=deftheta

if keyword_set(phi) then begin
   if phi[0] gt phi[1] then begin
      print,''
      dprint, 'Error with phi input. Phi angles must be specified in ascending order.'
      return
   endif
   if (phi[1] - phi[0]) gt 360 then begin
      ;if angle eq 'phi' || angle eq 'theta' then wrapphi = phi[1] - phi[0] - 360
      wrapphi = phi[1] - phi[0] - 360
      phi[1] = phi[0] + 360
   endif
endif else phi=defphi

if keyword_set(pitch) then begin
   if pitch[1] lt pitch[0] OR (pitch[0] lt 0 OR pitch[1] gt 180) then begin
      print,''
      dprint, 'Error with pitch input. Does not satisfy: 0 < pitch[0] < pitch[1] < 180.'
      return
   endif
endif else pitch=defpitch

if keyword_set(gyro) then begin
   if gyro[0] gt gyro[1] then begin
      print,''
      dprint, 'Error with gyro input. Gyrophase angles must be specified in ascending order.'
      return
   endif
   if angle eq 'gyro' || angle eq 'pa' then begin
      if (gyro[1] - gyro[0]) gt 360 then begin
         wrapphi = gyro[1] - gyro[0] - 360
         gyro[1] = gyro[0] + 360
      endif
   endif
endif else begin
   gyro=defgyro
   if keyword_set(angle) then begin
      if angle eq 'gyro' || angle eq 'pa' then wrapphi = 0
   endif
endelse

; Check for type of spectra requested (e.g. angle or energy)
if ~ keyword_set(angle) AND ~ keyword_set(energy) then begin
   angle = def_angle
   dprint, 'setting angle data_type: ', angle
   energy=1
endif else if ~ keyword_set(angle) then angle = 0

if keyword_set(erange) eq 0 then erange=deferange
if keyword_set(data_type) eq 0 then begin
   data_type = defdata_type
   print,''
   dprint, 'setting default data_type: ', data_type
   print,''
endif

theta1 = min(theta)
theta2 = max(theta)
phi1 = phi[0]
phi2 = phi[1]

tr = time_double(trange)
ndays = (tr[1] - tr[0]) / 86400
timespan,tr[0],ndays

if keyword_set(bins2mask) then mask=1

if keyword_set(mask) then begin
   if keyword_set(bins2mask) then begin
      if array_equal(bins2mask, -1) then begin
         dprint,''
         dprint,'WARNING: bins2mask array is empty. Not masking any SST angle bins.'
         dprint,''
         mask=0
      endif else begin
         dprint,'Masking the following SST angle bins: ', fix(bins2mask)
      endelse
   endif else begin
      dprint,''
      dprint,'WARNING: bins2mask not set. Masked bins will be whatever '
      dprint,'default may or may not be set in thm_sst_ps**.pro.'
      dprint,''
   endelse
endif

datatype = data_type

; check for sst data_type requests and load sst data
sst_ind = where(strmid(datatype,1,1) eq 's', sst_count)
if sst_count ne 0 then begin
   sst_type = datatype[sst_ind]
   thm_load_sst, probe=probe, trange=trange, datatype=sst_type, $
                get_support_data=get_support_data
endif

; check for esa data_type requests and load esa data
esa_ind = where(strmid(datatype,1,1) eq 'e', esa_count)
if esa_count ne 0 then begin
   esa_type = datatype[esa_ind]
   thm_load_esa_pkt, probe=probe, trange=trange, datatype=esa_type, $
                    get_support_data=get_support_data
endif


if strcmp(angle,'pa') || strcmp(angle,'gyro') then begin
   if ~ keyword_set(regrid) then begin
      dprint, 'REGRID keyword not set. Defaulting to [16 phis x 8 thetas].
      regrid=[16,8]
   endif

   ; move phi range to between 0-360
   if ~ ((phi1 le 360 && phi1 ge 0) && (phi2 le 360 && phi2 ge 0)) then begin
      if phi2 lt (phi2 - phi1) then begin
         phi1_1 = 360 + phi1
         phi1_2 = 360
         phi2_1 = 0
         phi2_2 = phi2
      endif
      if phi2 gt (phi2 - phi1) then begin
         phi1_1 = phi1
         phi1_2 = 360
         phi2_1 = 0
         phi2_2 = phi2 - 360
      endif
   endif else begin
      phi1_1 = phi1
      phi1_2 = phi2
      phi2_1 = phi1
      phi2_2 = phi2
   endelse
   phi[0] = min([phi1_1, phi1_2, phi2_1, phi2_2])
   phi[1] = max([phi1_1, phi1_2, phi2_1, phi2_2])

   coord = 'dsl'
   mag_suffix = '_fgs_'+coord+'_'+angle
   thm_load_state, probe=probe, trange=trange, /get_support_data
   thm_load_fgm, probe=probe, trange=trange, datatype='fgs', suffix='_'+angle, $
                coord='dsl', get_support_data=get_support_data
endif

; Calc and create spectra tplot variables
thm_part_moments2, probe=probe, instruments_types=datatype, trange=trange, $
                  tplotnames=tn, theta=theta, phi=phi, pitch=pitch, $
                  erange=erange, tplotsuffix=suffix, start_angle=start_angle, $
                  doangle=angle, doenergy=energy, wrapphi=wrapphi, $
                  mask=mask, mag_suffix=mag_suffix, regrid=regrid, $
                  gyro=gyro, other_dim=other_dim, en_tnames=en_tnames, $
                  an_tnames=an_tnames, normalize=normalize, $
                  bins2mask=bins2mask, _extra=ex, test=test


; Setup angle/energy spectrum tplot variables calculated by thm_part_moments2

if keyword_set(angle) then begin
   stheta = strcompress(string(theta))
   sphi = strcompress(string(phi))
   serang = strcompress(string(erange))
   spitch = strcompress(string(pitch))
   sgyro = strcompress(string(gyro))
   title = strjoin(['theta=', stheta, ', phi=', sphi, ', erange=', serang])
   if angle eq 'gyro' || angle eq 'pa' then begin
      title = strjoin(['theta=', stheta, ', phi=', sphi, ', pitch=', spitch, $
                    ', gyro=', sgyro, ', erange=', serang])
   endif
   tplot_options,'title',''
   tplot_options,'xmargin',[15,10]
   options,an_tnames,'y_no_interp',1
   options,an_tnames,'x_no_interp',1
   options,an_tnames,'ztitle','Eflux [eV/cm2/s/ster/eV]'
   options,an_tnames,minzlog=1
   zlim,an_tnames,1,1,1
;   ylim,'th*an_eflux*',p_start_angle, p_end_angle,log=0
   ;tdegap, 'th*an_eflux', overwrite=1
endif

if keyword_set(energy) then begin
   stheta = strcompress(string(theta))
   sphi = strcompress(string(phi))
   serang = strcompress(string(erange))
   title = strjoin(['theta=', stheta,', phi=',sphi,', erange=', $
                    serang])
   tplot_options,'title',''
   tplot_options,'xmargin',[15,10]
   options,en_tnames,'y_no_interp',1
   options,en_tnames,'x_no_interp',1
   options,en_tnames,'ztitle','Eflux [eV/cm2/s/ster/eV]'
   options,en_tnames,minzlog=1
   zlim,en_tnames,1,1,1
   options,en_tnames,ystyle=1
endif

; Autoplot the angle/energy spectra
if keyword_set(autoplot) && keyword_set(angle) then begin
   tplot_options,'title',title
   tplot,an_tnames
endif
if keyword_set(autoplot) && keyword_set(energy) then begin
   if keyword_set(angle) then begin
      tplot_options,'title',title
      tplot, an_tnames

      ; look for & plot nrg spectra vars w/more than 1 nrg channel
      for i=0,size(en_tnames,/n_elements)-1 do begin
         get_data,en_tnames[i],x,y,v
         if size(v,/n_dimensions) le 1 then begin
            dprint, en_tnames[i]," has only 1 energy channel.  Can't plot ", $
                    'spectra unless ERANGE is larger."
            continue
         endif else begin
            tplot, en_tnames[i], /ADD
         endelse
      endfor

   endif else begin
      tplot_options,'title',title

      ; look for & plot nrg spectra vars w/more than 1 nrg channel
      for i=0,size(en_tnames,/n_elements)-1 do begin
         get_data,en_tnames[i],x,y,v
         if size(v,/n_dimensions) le 1 then begin
            dprint, en_tnames[i]," has only 1 energy channel.  Can't plot ", $
                    'spectra unless ERANGE is larger."
            continue
         endif else begin
            tplot, en_tnames[i], /ADD
         endelse
      endfor

   endelse
endif
end